package at.ac.uibk.gitsearch.web.rest;

import at.ac.uibk.gitsearch.GitsearchApp;
import at.ac.uibk.gitsearch.domain.WatchListEntry;
import at.ac.uibk.gitsearch.domain.UserWatchList;
import at.ac.uibk.gitsearch.repository.WatchListEntryRepository;
import at.ac.uibk.gitsearch.repository.search.WatchListEntrySearchRepository;
import at.ac.uibk.gitsearch.service.WatchListEntryService;
import at.ac.uibk.gitsearch.service.dto.WatchListEntryDTO;
import at.ac.uibk.gitsearch.service.mapper.WatchListEntryMapper;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.mockito.Mock;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.junit.jupiter.MockitoExtension;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.PageRequest;
import org.springframework.http.MediaType;
import org.springframework.security.test.context.support.WithMockUser;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.transaction.annotation.Transactional;
import javax.persistence.EntityManager;
import java.util.Collections;
import java.util.List;

import static org.assertj.core.api.Assertions.assertThat;
import static org.elasticsearch.index.query.QueryBuilders.queryStringQuery;
import static org.hamcrest.Matchers.hasItem;
import static org.mockito.Mockito.*;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.*;

/**
 * Integration tests for the {@link WatchListEntryResource} REST controller.
 */
@SpringBootTest(classes = GitsearchApp.class)
@ExtendWith(MockitoExtension.class)
@AutoConfigureMockMvc
@WithMockUser
public class WatchListEntryResourceIT {

    private static final String DEFAULT_EXERCISE_ID = "AAAAAAAAAA";
    private static final String UPDATED_EXERCISE_ID = "BBBBBBBBBB";

    private static final String DEFAULT_EXERCISE_NAME = "AAAAAAAAAA";
    private static final String UPDATED_EXERCISE_NAME = "BBBBBBBBBB";

    @Autowired
    private WatchListEntryRepository watchListEntryRepository;

    @Autowired
    private WatchListEntryMapper watchListEntryMapper;

    @Autowired
    private WatchListEntryService watchListEntryService;

    /**
     * This repository is mocked in the at.ac.uibk.gitsearch.repository.search test package.
     *
     * @see at.ac.uibk.gitsearch.repository.search.WatchListEntrySearchRepositoryMockConfiguration
     */
    @Autowired
    private WatchListEntrySearchRepository mockWatchListEntrySearchRepository;

    @Autowired
    private EntityManager em;

    @Autowired
    private MockMvc restWatchListEntryMockMvc;

    private WatchListEntry watchListEntry;

    /**
     * Create an entity for this test.
     *
     * This is a static method, as tests for other entities might also need it,
     * if they test an entity which requires the current entity.
     */
    public static WatchListEntry createEntity(EntityManager em) {
        WatchListEntry watchListEntry = new WatchListEntry()
            .exerciseId(DEFAULT_EXERCISE_ID)
            .exerciseName(DEFAULT_EXERCISE_NAME);
        // Add required entity
        UserWatchList userWatchList;
        if (TestUtil.findAll(em, UserWatchList.class).isEmpty()) {
            userWatchList = UserWatchListResourceIT.createEntity(em);
            em.persist(userWatchList);
            em.flush();
        } else {
            userWatchList = TestUtil.findAll(em, UserWatchList.class).get(0);
        }
        watchListEntry.setWatchlistId(userWatchList);
        return watchListEntry;
    }
    /**
     * Create an updated entity for this test.
     *
     * This is a static method, as tests for other entities might also need it,
     * if they test an entity which requires the current entity.
     */
    public static WatchListEntry createUpdatedEntity(EntityManager em) {
        WatchListEntry watchListEntry = new WatchListEntry()
            .exerciseId(UPDATED_EXERCISE_ID)
            .exerciseName(UPDATED_EXERCISE_NAME);
        // Add required entity
        UserWatchList userWatchList;
        if (TestUtil.findAll(em, UserWatchList.class).isEmpty()) {
            userWatchList = UserWatchListResourceIT.createUpdatedEntity(em);
            em.persist(userWatchList);
            em.flush();
        } else {
            userWatchList = TestUtil.findAll(em, UserWatchList.class).get(0);
        }
        watchListEntry.setWatchlistId(userWatchList);
        return watchListEntry;
    }

    @BeforeEach
    public void initTest() {
        watchListEntry = createEntity(em);
    }

    @Test
    @Transactional
    public void createWatchListEntry() throws Exception {
        int databaseSizeBeforeCreate = watchListEntryRepository.findAll().size();
        // Create the WatchListEntry
        WatchListEntryDTO watchListEntryDTO = watchListEntryMapper.toDto(watchListEntry);
        restWatchListEntryMockMvc.perform(post("/api/watch-list-entries")
            .contentType(MediaType.APPLICATION_JSON)
            .content(TestUtil.convertObjectToJsonBytes(watchListEntryDTO)))
            .andExpect(status().isCreated());

        // Validate the WatchListEntry in the database
        List<WatchListEntry> watchListEntryList = watchListEntryRepository.findAll();
        assertThat(watchListEntryList).hasSize(databaseSizeBeforeCreate + 1);
        WatchListEntry testWatchListEntry = watchListEntryList.get(watchListEntryList.size() - 1);
        assertThat(testWatchListEntry.getExerciseId()).isEqualTo(DEFAULT_EXERCISE_ID);
        assertThat(testWatchListEntry.getExerciseName()).isEqualTo(DEFAULT_EXERCISE_NAME);

        // Validate the WatchListEntry in Elasticsearch
        verify(mockWatchListEntrySearchRepository, times(1)).save(testWatchListEntry);
    }

    @Test
    @Transactional
    public void createWatchListEntryWithExistingId() throws Exception {
        int databaseSizeBeforeCreate = watchListEntryRepository.findAll().size();

        // Create the WatchListEntry with an existing ID
        watchListEntry.setId(1L);
        WatchListEntryDTO watchListEntryDTO = watchListEntryMapper.toDto(watchListEntry);

        // An entity with an existing ID cannot be created, so this API call must fail
        restWatchListEntryMockMvc.perform(post("/api/watch-list-entries")
            .contentType(MediaType.APPLICATION_JSON)
            .content(TestUtil.convertObjectToJsonBytes(watchListEntryDTO)))
            .andExpect(status().isBadRequest());

        // Validate the WatchListEntry in the database
        List<WatchListEntry> watchListEntryList = watchListEntryRepository.findAll();
        assertThat(watchListEntryList).hasSize(databaseSizeBeforeCreate);

        // Validate the WatchListEntry in Elasticsearch
        verify(mockWatchListEntrySearchRepository, times(0)).save(watchListEntry);
    }


    @Test
    @Transactional
    public void checkExerciseIdIsRequired() throws Exception {
        int databaseSizeBeforeTest = watchListEntryRepository.findAll().size();
        // set the field null
        watchListEntry.setExerciseId(null);

        // Create the WatchListEntry, which fails.
        WatchListEntryDTO watchListEntryDTO = watchListEntryMapper.toDto(watchListEntry);


        restWatchListEntryMockMvc.perform(post("/api/watch-list-entries")
            .contentType(MediaType.APPLICATION_JSON)
            .content(TestUtil.convertObjectToJsonBytes(watchListEntryDTO)))
            .andExpect(status().isBadRequest());

        List<WatchListEntry> watchListEntryList = watchListEntryRepository.findAll();
        assertThat(watchListEntryList).hasSize(databaseSizeBeforeTest);
    }

    @Test
    @Transactional
    public void getAllWatchListEntries() throws Exception {
        // Initialize the database
        watchListEntryRepository.saveAndFlush(watchListEntry);

        // Get all the watchListEntryList
        restWatchListEntryMockMvc.perform(get("/api/watch-list-entries?sort=id,desc"))
            .andExpect(status().isOk())
            .andExpect(content().contentType(MediaType.APPLICATION_JSON_VALUE))
            .andExpect(jsonPath("$.[*].id").value(hasItem(watchListEntry.getId().intValue())))
            .andExpect(jsonPath("$.[*].exerciseId").value(hasItem(DEFAULT_EXERCISE_ID)))
            .andExpect(jsonPath("$.[*].exerciseName").value(hasItem(DEFAULT_EXERCISE_NAME)));
    }
    
    @Test
    @Transactional
    public void getWatchListEntry() throws Exception {
        // Initialize the database
        watchListEntryRepository.saveAndFlush(watchListEntry);

        // Get the watchListEntry
        restWatchListEntryMockMvc.perform(get("/api/watch-list-entries/{id}", watchListEntry.getId()))
            .andExpect(status().isOk())
            .andExpect(content().contentType(MediaType.APPLICATION_JSON_VALUE))
            .andExpect(jsonPath("$.id").value(watchListEntry.getId().intValue()))
            .andExpect(jsonPath("$.exerciseId").value(DEFAULT_EXERCISE_ID))
            .andExpect(jsonPath("$.exerciseName").value(DEFAULT_EXERCISE_NAME));
    }
    @Test
    @Transactional
    public void getNonExistingWatchListEntry() throws Exception {
        // Get the watchListEntry
        restWatchListEntryMockMvc.perform(get("/api/watch-list-entries/{id}", Long.MAX_VALUE))
            .andExpect(status().isNotFound());
    }

    @Test
    @Transactional
    public void updateWatchListEntry() throws Exception {
        // Initialize the database
        watchListEntryRepository.saveAndFlush(watchListEntry);

        int databaseSizeBeforeUpdate = watchListEntryRepository.findAll().size();

        // Update the watchListEntry
        WatchListEntry updatedWatchListEntry = watchListEntryRepository.findById(watchListEntry.getId()).get();
        // Disconnect from session so that the updates on updatedWatchListEntry are not directly saved in db
        em.detach(updatedWatchListEntry);
        updatedWatchListEntry
            .exerciseId(UPDATED_EXERCISE_ID)
            .exerciseName(UPDATED_EXERCISE_NAME);
        WatchListEntryDTO watchListEntryDTO = watchListEntryMapper.toDto(updatedWatchListEntry);

        restWatchListEntryMockMvc.perform(put("/api/watch-list-entries")
            .contentType(MediaType.APPLICATION_JSON)
            .content(TestUtil.convertObjectToJsonBytes(watchListEntryDTO)))
            .andExpect(status().isOk());

        // Validate the WatchListEntry in the database
        List<WatchListEntry> watchListEntryList = watchListEntryRepository.findAll();
        assertThat(watchListEntryList).hasSize(databaseSizeBeforeUpdate);
        WatchListEntry testWatchListEntry = watchListEntryList.get(watchListEntryList.size() - 1);
        assertThat(testWatchListEntry.getExerciseId()).isEqualTo(UPDATED_EXERCISE_ID);
        assertThat(testWatchListEntry.getExerciseName()).isEqualTo(UPDATED_EXERCISE_NAME);

        // Validate the WatchListEntry in Elasticsearch
        verify(mockWatchListEntrySearchRepository, times(1)).save(testWatchListEntry);
    }

    @Test
    @Transactional
    public void updateNonExistingWatchListEntry() throws Exception {
        int databaseSizeBeforeUpdate = watchListEntryRepository.findAll().size();

        // Create the WatchListEntry
        WatchListEntryDTO watchListEntryDTO = watchListEntryMapper.toDto(watchListEntry);

        // If the entity doesn't have an ID, it will throw BadRequestAlertException
        restWatchListEntryMockMvc.perform(put("/api/watch-list-entries")
            .contentType(MediaType.APPLICATION_JSON)
            .content(TestUtil.convertObjectToJsonBytes(watchListEntryDTO)))
            .andExpect(status().isBadRequest());

        // Validate the WatchListEntry in the database
        List<WatchListEntry> watchListEntryList = watchListEntryRepository.findAll();
        assertThat(watchListEntryList).hasSize(databaseSizeBeforeUpdate);

        // Validate the WatchListEntry in Elasticsearch
        verify(mockWatchListEntrySearchRepository, times(0)).save(watchListEntry);
    }

    @Test
    @Transactional
    public void deleteWatchListEntry() throws Exception {
        // Initialize the database
        watchListEntryRepository.saveAndFlush(watchListEntry);

        int databaseSizeBeforeDelete = watchListEntryRepository.findAll().size();

        // Delete the watchListEntry
        restWatchListEntryMockMvc.perform(delete("/api/watch-list-entries/{id}", watchListEntry.getId())
            .accept(MediaType.APPLICATION_JSON))
            .andExpect(status().isNoContent());

        // Validate the database contains one less item
        List<WatchListEntry> watchListEntryList = watchListEntryRepository.findAll();
        assertThat(watchListEntryList).hasSize(databaseSizeBeforeDelete - 1);

        // Validate the WatchListEntry in Elasticsearch
        verify(mockWatchListEntrySearchRepository, times(1)).deleteById(watchListEntry.getId());
    }

    @Test
    @Transactional
    public void searchWatchListEntry() throws Exception {
        // Configure the mock search repository
        // Initialize the database
        watchListEntryRepository.saveAndFlush(watchListEntry);
        when(mockWatchListEntrySearchRepository.search(queryStringQuery("id:" + watchListEntry.getId()), PageRequest.of(0, 20)))
            .thenReturn(new PageImpl<>(Collections.singletonList(watchListEntry), PageRequest.of(0, 1), 1));

        // Search the watchListEntry
        restWatchListEntryMockMvc.perform(get("/api/_search/watch-list-entries?query=id:" + watchListEntry.getId()))
            .andExpect(status().isOk())
            .andExpect(content().contentType(MediaType.APPLICATION_JSON_VALUE))
            .andExpect(jsonPath("$.[*].id").value(hasItem(watchListEntry.getId().intValue())))
            .andExpect(jsonPath("$.[*].exerciseId").value(hasItem(DEFAULT_EXERCISE_ID)))
            .andExpect(jsonPath("$.[*].exerciseName").value(hasItem(DEFAULT_EXERCISE_NAME)));
    }
}
